@@ -1,22 +1,23 @@
+{-# LANGUAGE CPP #-}
+
+#if __GLASGOW_HASKELL__ >= 902
+{-# OPTIONS_GHC -Wno-incomplete-uni-patterns #-}
+#endif
+
 -- | This is the code for the main engine.  This captures the posix
 -- subexpressions.  There is also a non-capturing engine, and a
 -- testing engine.
--- 
+--
 -- It is polymorphic over the internal Uncons type class, and
 -- specialized to produce the needed variants.
+
 module Text.Regex.TDFA.NewDFA.Engine_FA(execMatch) where
 
 import Data.Array.Base(unsafeRead,unsafeWrite,STUArray(..))
--- #ifdef __GLASGOW_HASKELL__
+
 import GHC.Arr(STArray(..))
 import GHC.ST(ST(..))
 import GHC.Exts(MutableByteArray#,RealWorld,Int#,sizeofMutableByteArray#,unsafeCoerce#)
-{-
--- #else
-import Control.Monad.ST(ST)
-import Data.Array.ST(STArray)
--- #endif
--}
 
 import Prelude hiding ((!!))
 import Control.Monad(when,unless,forM,forM_,liftM2,foldM)
@@ -28,6 +29,7 @@ import Data.IntMap(IntMap)
 import qualified Data.IntMap as IMap(null,toList,lookup,insert)
 import Data.Maybe(catMaybes)
 import Data.Monoid as Mon(Monoid(..))
+import Data.IntSet(IntSet)
 import qualified Data.IntSet as ISet(toAscList,null)
 import Data.Array.IArray((!))
 import Data.List(sortBy,groupBy)
@@ -60,12 +62,12 @@ set = unsafeWrite
 
 noSource :: ((Index, Instructions),STUArray s Tag Position,OrbitLog)
 noSource = ((-1,err "noSource"),err "noSource",err "noSource")
- 
+
 {-# SPECIALIZE execMatch :: Regex -> Position -> Char -> ([] Char) -> [MatchArray] #-}
 {-# SPECIALIZE execMatch :: Regex -> Position -> Char -> (Seq Char) -> [MatchArray] #-}
 {-# SPECIALIZE execMatch :: Regex -> Position -> Char -> SBS.ByteString -> [MatchArray] #-}
 {-# SPECIALIZE execMatch :: Regex -> Position -> Char -> LBS.ByteString -> [MatchArray] #-}
-execMatch :: Uncons text => Regex -> Position -> Char -> text -> [MatchArray]
+execMatch :: forall text. Uncons text => Regex -> Position -> Char -> text -> [MatchArray]
 execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
                  , regex_init = startState
                  , regex_b_index = b_index
@@ -81,12 +83,13 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
   orbitTags :: [Tag]
   !orbitTags = map fst . filter ((Orbit==).snd) . assocs $ aTags
 
-  !test = mkTest newline         
+  test :: WhichTest -> Index -> Char -> text -> Bool
+  !test = mkTest newline
 
   comp :: C s
   comp = {-# SCC "matchHere.comp" #-} ditzyComp'3 aTags
 
-  goNext :: ST s [MatchArray]
+  goNext :: forall s. ST s [MatchArray]
   goNext = {-# SCC "goNext" #-} do
     (SScratch s1In s2In (winQ,blank,which)) <- newScratch b_index b_tags
     spawnAt b_tags blank startState s1In offsetIn
@@ -170,6 +173,7 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
 -- The updated Orbits get the new ordinal value and an empty (Seq
 -- Position).
 
+        compressOrbits :: MScratch s -> IntSet -> Position -> ST s ()
         compressOrbits s1 did offset = do
           let getStart state = do start <- maybe (err "compressOrbit,1") (!! 0) =<< m_pos s1 !! state
                                   return (state,start)
@@ -184,7 +188,7 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
                                     _ -> return Nothing )
                 let compressGroup [((state,_),orbit)] | Seq.null (getOrbits orbit) = return ()
                                                       | otherwise =
-                      set (m_orbit s1) state 
+                      set (m_orbit s1) state
                       . (IMap.insert tag $! (orbit { ordinal = Nothing, getOrbits = mempty}))
                       =<< m_orbit s1 !! state
 
@@ -214,6 +218,17 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
 -- "storeNext".  If no winners are ready to be released then the
 -- computation continues immediately.
 
+        findTrans
+          :: MScratch s
+          -> MScratch s
+          -> IntSet
+          -> SetIndex
+          -> DT
+          -> DTrans
+          -> Index
+          -> Char
+          -> text
+          -> ST s [MatchArray]
         findTrans s1 s2 did did' dt' dtrans offset prev' input' =  {-# SCC "goNext.findTrans" #-} do
           -- findTrans part 0
           -- MAGIC TUNABLE CONSTANT 100 (and 100-1). TODO: (offset .&. 127 == 127) instead?
@@ -231,7 +246,8 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
                     challenge x1@((_si1,ins1),_p1,_o1) x2@((_si2,ins2),_p2,_o2) = {-# SCC "goNext.findTrans.challenge" #-} do
                       check <- comp offset x1 (newPos ins1) x2 (newPos ins2)
                       if check==LT then return x2 else return x1
-                (first:rest) <- mapM prep (IMap.toList sources)
+                first_rest <- mapM prep (IMap.toList sources)
+                let first:rest = first_rest
                 set which destIndex =<< foldM challenge first rest
           let dl = IMap.toList dtrans
           mapM_ findTransTo dl
@@ -256,6 +272,7 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
 -- (futher test "(.+|.+.)*" on "aa\n")
 
         {-# INLINE processWinner #-}
+        processWinner :: MScratch s -> IntMap Instructions -> Position -> ST s ()
         processWinner s1 w offset = {-# SCC "goNext.newWinnerThenProceed" #-} do
           let prep x@(sourceIndex,instructions) = {-# SCC "goNext.newWinnerThenProceed.prep" #-} do
                 pos <- maybe (err "newWinnerThenProceed,1") return =<< m_pos s1 !! sourceIndex
@@ -271,12 +288,14 @@ execMatch (Regex { regex_dfa =  DFA {d_id=didIn,d_dt=dtIn}
             [] -> return ()
             (first:rest) -> newWinner offset =<< foldM challenge first rest
 
+        newWinner :: Position -> ((a, Instructions), STUArray s Tag Position, c) -> ST s ()
         newWinner preTag ((_sourceIndex,winInstructions),oldPos,_newOrbit) = {-# SCC "goNext.newWinner" #-} do
           newerPos <- newA_ b_tags
           copySTU oldPos newerPos
           doActions preTag newerPos (newPos winInstructions)
           putMQ (WScratch newerPos) winQ
 
+        finalizeWinner :: ST s [MatchArray]
         finalizeWinner = do
           mWinner <- readSTRef (mq_mWin winQ)
           case mWinner of
@@ -357,7 +376,7 @@ showMS s i = do
 showWS :: WScratch s -> ST s String
 showWS (WScratch pos) = do
   a <- getAssocs pos
-  return $ unlines [ "WScratch" 
+  return $ unlines [ "WScratch"
                    , indent (show a)]
 -}
 {- CREATING INITIAL MUTABLE SCRATCH DATA STRUCTURES -}
@@ -419,7 +438,7 @@ ditzyComp'3 aTagOP = comp0 where
 
   allcomps :: Tag -> [F s]
   allcomps tag | tag > top = [F (\ _ _ _ _ _ _ -> return EQ)]
-               | otherwise = 
+               | otherwise =
     case aTagOP ! tag of
       Orbit -> F (challenge_Orb tag) : allcomps (succ tag)
       Maximize -> F (challenge_Max tag) : allcomps (succ tag)
@@ -427,6 +446,15 @@ ditzyComp'3 aTagOP = comp0 where
       Minimize -> err "allcomps Minimize"
    where top = snd (bounds aTagOP)
 
+  challenge_Ignore
+    :: Int
+    -> [F s1]
+    -> Position
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ST s1 Ordering
   challenge_Ignore !tag (F next:comps) preTag x1 np1 x2 np2 =
     case np1 of
       ((t1,_):rest1) | t1==tag ->
@@ -439,6 +467,15 @@ ditzyComp'3 aTagOP = comp0 where
           _ ->  next comps preTag x1 np1 x2 np2
   challenge_Ignore _ [] _ _ _ _ _ = err "impossible 2347867"
 
+  challenge_Max
+    :: Int
+    -> [F s1]
+    -> Position
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ST s1 Ordering
   challenge_Max !tag (F next:comps) preTag x1@(_state1,pos1,_orbit1') np1 x2@(_state2,pos2,_orbit2') np2 =
     case np1 of
       ((t1,b1):rest1) | t1==tag ->
@@ -468,7 +505,16 @@ ditzyComp'3 aTagOP = comp0 where
               else return (compare p1 p2)
   challenge_Max _ [] _ _ _ _ _ = err "impossible 9384324"
 
-  challenge_Orb !tag (F next:comps) preTag x1@(_state1,_pos1,orbit1') np1 x2@(_state2,_pos2,orbit2') np2 = 
+  challenge_Orb
+    :: Int
+    -> [F s1]
+    -> Position
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ((Int, Instructions), STUArray s1 Tag Position, IntMap Orbits)
+    -> [(Int, Action)]
+    -> ST s1 Ordering
+  challenge_Orb !tag (F next:comps) preTag x1@(_state1,_pos1,orbit1') np1 x2@(_state2,_pos2,orbit2') np2 =
     let s1 = IMap.lookup tag orbit1'
         s2 = IMap.lookup tag orbit2'
     in case (s1,s2) of
@@ -489,7 +535,7 @@ comparePos :: (ViewL Position) -> (ViewL Position) -> Ordering
 comparePos EmptyL EmptyL = EQ
 comparePos EmptyL _      = GT
 comparePos _      EmptyL = LT
-comparePos (p1 :< ps1) (p2 :< ps2) = 
+comparePos (p1 :< ps1) (p2 :< ps2) =
   compare p1 p2 `mappend` comparePos (viewl ps1) (viewl ps2)
 
 {- CONVERT WINNERS TO MATCHARRAY -}
@@ -576,7 +622,7 @@ copySTU _souce@(STUArray _ _ _ msource) _destination@(STUArray _ _ _ mdest) =
     case unsafeCoerce# memcpy mdest msource n# s1# of { (# s2#, () #) ->
     (# s2#, () #) }}
 {-
-#else /* !__GLASGOW_HASKELL__ */
+-- #else /* !__GLASGOW_HASKELL__ */
 
 copySTU :: (MArray (STUArray s) e (S.ST s))=> STUArray s Tag e -> STUArray s Tag e -> S.ST s (STUArray s i e)
 copySTU source destination = do
@@ -587,5 +633,5 @@ copySTU source destination = do
   forM_ (range b) $ \index ->
     set destination index =<< source !! index
   return destination
-#endif /* !__GLASGOW_HASKELL__ */
+-- #endif /* !__GLASGOW_HASKELL__ */
 -}
